// -- File: NAVIGATE.CC

CLASS Navigate
/* ----------------------------------------------------------------------
    Description:
 
       This class contains a set of navigation-oriented
       routines. (Note to be confused with navigation in
       a table, this refers to navigation using a sailboat
       or some other means of travel ...)
 
    Programmers: 
 
       Ken Mayer
       Jay Parsons 
 
    History: 
 
       11/25/1996 -- original version of these routines as a class.
 
    Instantiation: 
 
       set procedure to navigate.cc additive
       _app.Navigate = new Navigate()

    **** NOTE: If you do not use the release() method of Navigate,
         you will not be able to release the reference to the
         MathEx class that is created in the init() method.
 
    Methods:
       More details on each of these is contained in the commentary for
       each method ...
    
       Init()
         Initializes classes necessary
 
       Release()
         Releases this object (and others) from memory
 
       BearsDist()
         Distance to object at second bearing, given 2 bearings & 
         distance between
 
       BearsPass()
         Distance abeam at which to pass object, given 2 bearings & run
  
       BearsRun()
         Distance to run until object will be abeam, given 2 bearings
 
       Correct()
         Convert compass direction to magnetic or magnetic to true.
  
       Course()
         Course made good given heading, speed, wind direction & force
 
       GCCourse()
         Initial great circle course from one point to another
 
       GCDist()
         Great circle distance between two points on earth
 
       GndSpeed()
         Ground speed given heading, speed, wind direction and force
 
       Heading()
         Heading required to make good a course
 
       LeftWind()
         .T. if effect of second vector on first is from left
 
       TailWind()
         Whether effect of second vector is additive
 
       UnCorrect()
         Uncorrection of direction - adjusts direction
 
       XAngle()
         Angle in degrees at which two vectors intersect
 
   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           mathex.cc
   **           convert.cc
   **           setproc.prg
 
   -------------------------------------------------------------------- */ 

   class::Init()

   FUNCTION Init
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/10/1996
   *-- Notes.......: Ensures that routines needed are available
   *--               It is required that this be used (see instructions
   *--               above) for this class to function properly.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/10/1996 -- Original Code
   *-- Calls.......: mathex.cc
   *-- Usage.......: navigate.Init()
   *-- Example.....: _app.Navigate.Init()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------

      PROTECT MathProc
      set procedure to :dUFLP:SetProc.prg additive
      if not SetProc( ":dUFLP:MATHEX.CC" )
         set procedure to :dUFLP:mathex.cc additive
         this.MathProc = true
      else
         this.MathProc = false
      endif
      this.Math        = new MathEx()
      this.Math.parent = this
      set procedure to :dUFLP:convert.cc additive
      this.Convert = new Convert()
      
   RETURN NULL

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: navigate.Release()
   *-- Example.....: _app.Navigate.Release()
   *--               _app.navigate = NULL   && not required, but a good
   *--                                      && idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      this.math.release()
      this.math = null
      if this.MathProc
         close procedure :dUFLP:mathex.cc
      endif
      close procedure :dUFLP:Convert.cc

      release object this

   RETURN NULL
   *-- EoM: Release()

   FUNCTION BearsDist( nBear1, nBear2, nRun )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Distance to an object at the time of the second
   *--               bearing, given two bearings and the distance run
   *--               between them.  Value returned will be in same
   *--               units as third argument; first two are in degrees.
   *--               Returns -1 if already past the object.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.BearsDist(<nBear1>,<nBear2>,<nRun>)
   *-- Example.....: ? _app.Navigate.BearsDist(200,150,5)
   *-- Returns.....: Numeric = distance or time to object at time of 
   *--                         second bearing
   *-- Parameters..: nBear1 = First bearing of object
   *--               nBear2 = Second bearing of object
   *--               nRun   = Distance (or time) run between bearings
   *-------------------------------------------------------------------
      
      if nBear2 > 180
         if nBear1 < nBear2 or nBear2 < 270
            RETURN -1
         else
            nBear1 = 360 - nBear1
            nBear2 = 360 - nBear2
         endif
      else
         if nBear2 < nBear1 or nBear2 > 90
            RETURN -1
         endif
      endif

   RETURN sin( dtor( nBear1 ) ) * nRun / sin( dtor( nBear2 - nBear1 ) )
   *-- EoM: BearsDist()

   FUNCTION BearsPass( nBear1, nBear2, nRun )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Distance at which the object will be passed abeam:
   *--                                  * <-- Object
   *--                            .    /|
   *--                      .         / |
   *--                1-->-->-->-->--2--3
   *--               Where 1 = Position at time first bearing to object 
   *--                         is taken,
   *--                     2 = position at second bearing,
   *--                     3 = position at which the object will be 
   *--                         abeam.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.BearsPass(<nBear1>,<nBear2>,<nRun>)
   *-- Example.....: ? _app.Navigate.BearsPass(200,150,5)
   *-- Returns.....: Numeric  = distance or time to object when abeam
   *-- Parameters..: nBear1 = First bearing of object
   *--               nBear2 = Second bearing of object
   *--               nRun   = Distance (or time) run between bearings
   *-------------------------------------------------------------------
      
      local nTemp
      if nBear2 > 180
         if nBear1 < nBear2 or nBear2 < 270
            RETURN -1
         else
            nBear1 = 360 - nBear1
            nBear2 = 360 - nBear2
         endif
      else
         if nBear2 < nBear1 or nBear2 > 90
            RETURN -1
         endif
      endif
      nTemp = sin( dtor( nBear1 ) ) * ;
                 nRun / sin( dtor( nBear2 - nBear1 ) )
         
   RETURN nTemp * sin( dtor( nBear2 ) )
   *-- EoM: BearsPass()

   FUNCTION BearsRun( nBear1, nBear2, nRun )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Distance or time to run until object will be abeam
   *--               given two bearings and run.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.BearsRun(<nBear1>,<nBear2>,<nRun>)
   *-- Example.....: ? _app.Navigate.BearsRun(200,150,5)
   *-- Returns.....: numeric = distance or time until abeam of object
   *-- Parameters..: nBear1 = First bearing of object
   *--               nBear2 = Second bearing of object
   *--               nRun   = Distance (or time) run between bearings
   *-------------------------------------------------------------------
      
      local nTemp
      if nBear2 > 180
         if nBear1 < nBear2 or nBear2 < 270
            RETURN -1
         else
            nBear1 = 360 - nBear1
            nBear2 = 360 - nBear2
         endif
      else
         if nBear2 < nBear1 or nBear2 > 90
            RETURN -1
         endif
      endif
      nTemp = sin( dtor( nBear1 ) ) * ;
                  nRun / sin( dtor( nBear2 - nBear1 ) )
      
   RETURN nTemp * cos( dtor( nBear2 ) )
   *-- EoM: BearsRun()

   FUNCTION Correct( nDirection, xCorrection )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Correction of direction - adjusts direction given,
   *--               in degrees, by second number of degrees.  Use to 
   *--               convert a compass direction to magnetic using 
   *--               deviation as the second argument, or magnetic to 
   *--               true using variation as the second argument. 
   *--               Returns a direction in degrees.
   *--
   *--               A westerly second argument may be given either as 
   *--               a negative number or as a character value 
   *--               containing "W".  If second argument is character-
   *--               type but contains a negative value, effect of 
   *--               presence or absence of "W" is reversed.  That is, 
   *--               "-20 W" is treated like "20 E" or the number 20.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.   
   *-- Calls.......: None
   *-- Usage.......: navigate.Correct(<nDirection>,<xCorrection>)
   *-- Example.....: ? _app.Navigate.Correct(50,"-10 E")
   *-- Returns.....: Numeric (direction in degrees)
   *-- Parameters..: nDirection  = Heading
   *--               xCorrection = amount to 'correct' by, may be 
   *--                             numeric or character, see above 
   *--                             under 'Notes'.
   *-------------------------------------------------------------------
      
      local nCval
      
      if type( "xCorrection" ) = "C"
         nCVal = val( xCorrection )
         if "W" $ upper( xCorrection )
            nCVal = -nCVal
         endif
      else
         nCVal = xCorrection
      endif
      
   RETURN mod( 360 + nDirection + nCVal, 360 )
   *-- EoM: Correct()

   FUNCTION Course( nHeading, nSpeed, nWindFrom, nForce )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Course made good given heading, speed and 
   *--               direction and force of the wind or current.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: navigate.XAngle()
   *--               navigate.LeftWind()
   *--               navigate.TailWind()
   *-- Usage.......: navigate.Course(<nHeading>,<nSpeed>,<nWindFrom>,;
   *--                               <nForce>)
   *-- Example.....: ? _app.Navigate.Course(20,5,240,2)
   *-- Returns.....: Numeric (direction in degrees)
   *-- Parameters..: nHeading  = Direction of heading
   *--               nSpeed    = Speed through the air or water
   *--               nWindFrom = Direction wind or current is coming 
   *--                           from
   *--               nForce    = Force of the wind or current
   *-------------------------------------------------------------------
      
      local nTemp, nCrabAngle
      nTemp = dtor( CLASS::XAngle( nHeading, nWindFrom ) )
      nCrabAngle = nAirSpeed - nForce * cos( nTemp ) *;
                      iif( CLASS::TailWind( nHeading, nWindFrom ), -1, 1)
      if nCrabAngle < 0
         nCrabAngle = 0
      else
         nCrabAngle = abs( rtod( atan( nForce * sin( nTemp );
                         / nCrabAngle ) ) )
         nCrabAngle = iif( CLASS::LeftWind( nHeading, nWindFrom ),;
                           nCrabAngle, -nCrabAngle)
      endif
      
   RETURN mod( 360 + nHeading + nCrabAngle, 360 )
   *-- EoM: Course()

   FUNCTION GCCourse( cLat1, cLon1, cLat2, cLon2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Initial great circle course between two points 
   *--               given latitude and longitude of each.  Returns -1 
   *--               if the points are antipodes.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: convert.Deg2Num()
   *--               math.AHav()
   *--               math.CsCH()
   *--               math.Hav()
   *-- Usage.......: navigate.GCCourse(<cLat1>,<cLon1>,<cLat2>,<cLon2>)
   *-- Example.....: ? _app.Navigate.GCCourse(200,150,105,200)
   *-- Returns.....: Numeric (degrees)
   *-- Parameters..: cLat1 = Latitude 1
   *--               cLon1 = Longitude 1
   *--               cLat2 = Latitude 2
   *--               cLon2 = Longitude 2
   *-------------------------------------------------------------------
      
      local nLa1, nLo1, nLa2, nLo2, nDla, nDlo, nTemp, lRev
      
      nLa1 = dtor( this.Convert.Deg2Num( cLat1 ) )
      nLo1 = dtor( this.Convert.Deg2Num( cLon1 ) )
      nLa2 = dtor( this.Convert.Deg2Num( cLat2 ) )
      nLo2 = dtor( this.Convert.Deg2Num( cLon2 ) )
      nDla = abs( nLa1 - nLa2 )
      nDlo = abs( nLo2 - nLo1 )
      lRev = .F.
      do case
         case nDla =pi() or nDlo = pi () and nLa1 + nLa2 =0
              RETURN -1
         case nDlo =0 or nDlo = pi() or abs( nLa1) = pi() or;
              abs( nLa2 ) = pi()
              RETURN iif( La1 > La2 , 180, 0 )
         case nDlo > pi()
              nDlo = 2 * pi() - nDlo
              lRev = .T.
      endcase
      nTemp = this.Math.Hav( nDla ) + this.Math.Hav( nDlo ) * ;
              cos( nLa1 ) * cos( nLa2 )
      nTemp = rtod( asin( sin( nDlo ) * cos( nLa2 ) * ;
                this.Math.CsCH( this.Math.AHav( nTemp ) ) ) )
      nTemp = iif( nLa1 > nLa2, 180 - nTemp, nTemp )
      
   RETURN iif( ( nLo2 > nLo1 ) = lRev, nTemp, 360 - nTemp )
   *-- EoM: GCCourse()

   FUNCTION GCDist( cLat1, cLon1, cLat2, cLon2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Great circle distance between two points given 
   *--               latitude and longitude of each.  This function 
   *--               obtains the degrees of arc along the great circle 
   *--               and simply multiplies by 60 to convert the degrees 
   *--               to nautical miles.  As this ignores the 
   *--               eccentricity of the earth, the answer may be in 
   *--               error by approximately half of one percent.  In 
   *--               general, if the route lies close to the equator 
   *--               the result of this function will be smaller than 
   *--               the actual number of nautical miles, but if the 
   *--               route passes close to the poles the function 
   *--               result will be larger than the correct number.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: Convert.Deg2Num() 
   *--               math.AHav()
   *--               math.Hav()
   *-- Usage.......: navigate.GCDist(<cLat1>,<cLon1>,<cLat2>,<cLon2>)
   *-- Example.....: ? _app.Navigate.GCDist(200,150,105,200)
   *-- Returns.....: Numeric (nautical miles)
   *-- Parameters..: cLat1 = Latitude 1
   *--               cLon1 = Longitude 1
   *--               cLat2 = Latitude 2
   *--               cLon2 = Longitude 2
   *-------------------------------------------------------------------
                                                                           
      local nLa1, nLo1, nLa2, nLo2, nDla, nDlo, nTemp

      nLa1 = dtor( this.Convert.Deg2Num( cLat1 ) )
      nLo1 = dtor( this.Convert.Deg2Num( cLon1 ) )
      nLa2 = dtor( this.Convert.Deg2Num( cLat2 ) )
      nLo2 = dtor( this.Convert.Deg2Num( cLon2 ) )
      nDla = abs( nLa1 - nLa2 )
      nDlo = abs( nLo2 - nLo1 )
      do case
         case nDlo = 0 or nDla = pi()
              RETURN 60 * rtod( nDla )
         case nDlo = pi()
              RETURN 60 * rtod( ( pi() - nDla ) )
         case nDlo > pi()
              nDlo = 2 * pi() - nDlo
      endcase
      nTemp = this.Math.Hav( nDla ) + this.Math.Hav( nDlo ) * ;
              cos( nLa1 ) * cos( nLa2 )

   RETURN 60 * rtod( this.Math.AHav( nTemp ) )
   *-- EoM: GCDist()

   FUNCTION GndSpeed( nHeading, nSpeed, nWindFrom, nForce )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Speed over the ground or bottom given heading, 
   *--               speed and direction and force of the wind or 
   *--               current.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: navigate.XAngle()
   *--               navigate.TailWind()
   *-- Usage.......: navigate.GndSpeed(<nHeading>,<nSpeed>,;
   *--                                 <nWindFrom>,<nForce>)
   *-- Example.....: ? _app.Navigate.GndSpeed(20,5,240,2)
   *-- Returns.....: Numeric   = speed over ground or bottom
   *-- Parameters..: nHeading  = Direction of heading
   *--               nSpeed    = Speed through the air or water
   *--               nWindFrom = Direction wind or current is coming 
   *--                           from
   *--               nForce    = Force of the wind or current
   *-------------------------------------------------------------------
      
      local nTemp
      nTemp  = cos( dtor( CLASS::XAngle( nHeading, nWindFrom ) ) ) * ;
               iif( CLASS::TailWind( nHeading, nWindFrom ), -1, 1 )
      nTemp = nSpeed * nSpeed + nForce * nForce ;
                 - 2 * nSpeed * nForce * nTemp
      
   RETURN iif(nTemp<=0,nSpeed+nForce*iif(CLASS::TailWind(nHeading,;
              nWindFrom ),1,-1),sqrt(nTemp))
   *-- EoM: GndSpeed()

   FUNCTION Heading( nCourse, nSpeed, nWindFrom, nForce )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Heading required to make good a course.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: navigate.XAngle()
   *--               navigate.LeftWind()
   *-- Usage.......: navigate.Heading(<nCourse>,<nSpeed>,<nWindFrom>,;
   *--                                <nForce>)
   *-- Example.....: ? _app.Navigate.Heading(20,5,240,2)
   *-- Returns.....: Numeric (direction in degrees)
   *-- Parameters..: nCourse   = Direction of course to make good
   *--               nSpeed    = Speed through the air or water
   *--               nWindFrom = Direction wind or current is coming 
   *--                           from
   *--               nForce    = Force of the wind or current
   *-------------------------------------------------------------------
      
      local nCrabAngle
      nCrabAngle = rtod( asin( nForce * ;
                      sin( dtor( CLASS::XAngle( nCourse, nWindFrom)));
                      / nSpeed ) )
      nCrabAngle = iif( CLASS::LeftWind( nCourse, nWindFrom ),;
                           -nCrabAngle, nCrabAngle )
      nCrabAngle = mod( 360 + nCourse + nCrabAngle, 360 )
      
   RETURN iif( abs( nCrabAngle ) < 360, nCrabAngle, -1 )
   *-- EoM: Heading()

   FUNCTION LeftWind( nCourse, nWindFrom )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Whether effect of second vector on first is from 
   *--               the left or the right.  Returns .T. if from the 
   *--               left, else .F.  Expects vectors in degrees.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.LeftWind(<nCourse>,<nWindFrom>)
   *-- Example.....: ? _app.Navigate.LeftWind(20,240)
   *-- Returns.....: .T. if wind or current is from left, or .F.
   *-- Parameters..: nCourse   = Direction of course
   *--               nWindFrom = Direction wind or current is coming 
   *--                           from
   *-------------------------------------------------------------------
   
   RETURN ( nCourse > nWindfrom ) = ( abs( nCourse - nWindfrom ) < 180)
   *-- EoM: LeftWind()

   FUNCTION TailWind( nCourse, nWindFrom )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons 
   *-- Date........: 11/10/1996
   *-- Notes.......: Whether effect of second vector on first is 
   *--               additive or subtractive ( from behind or from 
   *--               ahead ).
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302 
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.TailWind(<nCourse>,<nWindFrom>)
   *-- Example.....: ? _app.Navigate.TailWind(20,240)
   *-- Returns.....: logical, .T. if wind/current is behind, or .F.
   *-- Parameters..: nCourse   = Direction of course
   *--               nWindFrom = Direction wind or current is coming 
   *--                           from
   *-------------------------------------------------------------------
   
   RETURN ( abs( abs( nCourse - nWindFrom ) - 180 ) < 90 )
   *-- EoM: TailWind()

   FUNCTION UnCorrect( nDirection, xUnCorr )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Uncorrection of direction - adjusts direction 
   *--               given, in degrees, by second number of degrees.  
   *--               The inverse of correct(), see above. Use to convert 
   *--               a true direction to magnetic using variation as 
   *--               the second argument, or magnetic to compass using 
   *--               deviation as the second argument.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.UnCorrect(<nDirection>,<xUnCorr>)
   *-- Example.....: ? _app.Navigate.UnCorrect(50,"10W")
   *-- Returns.....: Numeric (direction in degrees)
   *-- Parameters..: nDirection = Heading
   *--               xUnCorr    = amount to 'uncorrect' by, may be 
   *--                            numeric or character, see above 
   *--                            under 'Notes'.
   *-------------------------------------------------------------------
      
      local nCVal
      if type( "xUncorr" ) = "C"
         nCVal = val( xUncorr )
         if "W" $ upper( xUncorr )
            nCVal = -nCVal
         endif
      else
         nCVal = xUncorr
      endif
      
   RETURN mod( 360 + nDirection - nCVal, 360 )
   *-- EoM: UnCorrect()

   FUNCTION XAngle( nVector1, nVector2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons
   *-- Date........: 11/10/1996
   *-- Notes.......: Angle in degrees ( <= 90 ) at which two vectors in
   *--               degrees intersect.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: navigate.XAngle(<nVector1>,<nVector2>)
   *-- Example.....: ? _app.Navigate.XAngle(20,240)
   *-- Returns.....: Numeric = angle of intersection
   *-- Parameters..: nVector1 = First vector
   *--               nVector2 = Second vector
   *-------------------------------------------------------------------
      
      local nResult
      nResult = abs( nVector1 - nVector2)
      do case
         case nResult > 270
              nResult = 360 - nResult
         case nResult > 180
              nResult = nResult - 180
         case nResult > 90
              nResult = 180 - nResult
      endcase
      
   RETURN nResult
   *-- EoM: XAngle()

ENDCLASS
*-- End of Class: Navigate

// not sure what used to be in this file, but 
// believe this class does not require this file
// compile errors occur with this statement ...
//#INCLUDE <dUFLP.H>

/* -----------------------------------------------------------------------
   End of File: NAVIGATE.CC
   ----------------------------------------------------------------------- */
